/*
 * Copyright © 2011 Linaro Limited
 *
 * This file is part of glcompbench.
 *
 * glcompbench is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * glcompbench is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with glcompbench.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *  Alexandros Frantzis <alexandros.frantzis@linaro.org>
 *  Jesse Barker <jesse.barker@linaro.org>
 */

#ifndef COMPOSITE_WINDOW_PIXMAN_H_
#define COMPOSITE_WINDOW_PIXMAN_H_

#include <X11/Xlib.h>
#include <X11/extensions/XShm.h>
#include <sys/shm.h>
#include <pixman.h>

#include "composite-window.h"
#include "options.h"

class CompositeWindowPixman : public CompositeWindow
{
public:
    CompositeWindowPixman(Display *xdpy, Window win) :
        CompositeWindow(xdpy, win), pixman_image_(0), ximage_(0),
        use_shm_(have_xshm() && Options::use_accel_tfp)
        { shm_info_.shmaddr = reinterpret_cast<char *>(-1); }

    ~CompositeWindowPixman();

    CompositeWindow::Texture get_texture()
    {
        return CompositeWindow::Texture(pixman_image_);
    }

    bool update_texture();
    void update_texture_from_pixmap(bool recreate);

private:
    bool have_xshm();
    bool ensure_ximage();
    bool ensure_ximage_shm();
    bool ensure_pixman_image();
    void release_resources();

    pixman_image_t *pixman_image_;
    XImage *ximage_;
    bool use_shm_;
    XShmSegmentInfo shm_info_;
};

#endif
